@@ -31,6 +31,8 @@
 #include <ORBitservices/CosNaming_impl.h>
 #include <libbonobo.h>
 
+#include "bonobo-activation/bonobo-activation-private.h"
+
 #include "server.h"
 #include "activation-context.h"
 #include "activation-context-query.h"
@@ -39,51 +41,143 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <popt.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <locale.h>
 #include <string.h>
+#ifdef HAVE_SYSLOG_H
 #include <syslog.h>
+#endif
 
 #include <libxml/parser.h>
 
+#include <glib/gstdio.h>
+
+static gboolean        server_threaded = FALSE;
+static glong           server_guard_depth = 0;
+static GStaticRecMutex server_guard = G_STATIC_REC_MUTEX_INIT;
+
+PortableServer_POA
+server_get_poa (void)
+{
+        PortableServer_POA poa;
+
+        if (!g_getenv ("BONOBO_ACTIVATION_DISABLE_THREADING")) {
+                server_threaded = TRUE;
+                poa = bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST);
+        } else {
+                g_warning ("b-a-s running in non-threaded mode");
+                server_threaded = FALSE;
+                poa = bonobo_poa();
+        }
+        bonobo_activation_server_fork_init (server_threaded);
+
+        return poa;
+}
+
+ServerLockState
+server_lock_drop (void)
+{
+        glong i, state = server_guard_depth;
+
+        if (!server_threaded)
+                return 0;
+
 #ifdef BONOBO_ACTIVATION_DEBUG
-static void debug_queries (void);
+        fprintf (stderr, "thread %p dropping server guard with depth %ld\n",
+                 g_thread_self (), state);
 #endif
+        server_guard_depth = 0;
+        for (i = 0; i < state; i++)
+                g_static_rec_mutex_unlock (&server_guard);
+        return state;
+}
+
+void
+server_lock_resume (ServerLockState state)
+{
+        long i;
+
+        if (!server_threaded)
+                return;
+
+        for (i = 0; i < state; i++)
+                g_static_rec_mutex_lock (&server_guard);
+        server_guard_depth = state;
+#ifdef BONOBO_ACTIVATION_DEBUG
+        fprintf (stderr, "thread %p re-taken server guard with depth %ld\n",
+                 g_thread_self (), state);
+#endif
+}
+
+void
+server_lock (void)
+{
+        if (!server_threaded)
+                return;
+
+        g_static_rec_mutex_lock (&server_guard);
+        server_guard_depth++;
+        fprintf (stderr, "thread %p take guard [%ld]\n",
+                 g_thread_self (), server_guard_depth);
+}
+
+void
+server_unlock (void)
+{
+        if (!server_threaded)
+                return;
+
+        fprintf (stderr, "thread %p release guard [%ld]\n",
+                 g_thread_self (), server_guard_depth);
+        server_guard_depth--;
+        g_static_rec_mutex_unlock (&server_guard);
+}
+
+#ifdef G_OS_WIN32
+
+#undef SERVERINFODIR
+#define SERVERINFODIR _bonobo_activation_win32_get_serverinfodir ()
+
+#undef SERVER_LOCALEDIR
+#define SERVER_LOCALEDIR _bonobo_activation_win32_get_localedir ()
+
+#endif  /* Win32 */
 
 /* Option values */
-static char *od_source_dir = NULL;
+static gchar *od_source_dir = NULL;
 #ifdef BONOBO_ACTIVATION_DEBUG
-static char *ac_evaluate = NULL;
+static void debug_queries (void);
+static gchar *ac_evaluate = NULL;
+#endif
 static gboolean server_reg = FALSE;
 static gboolean output_debug = FALSE;
-#endif
-static int server_ac = 0, ior_fd = -1;
+static gboolean server_ac = FALSE;
+static gint ior_fd = -1;
 
-static struct poptOption options[] = {
+static const GOptionEntry options[] = {
 
-	{"od-source-dir", '\0', POPT_ARG_STRING, &od_source_dir, 0,
+	{"od-source-dir", '\0', 0, G_OPTION_ARG_FILENAME, &od_source_dir,
 	 N_("Directory to read .server files from"), N_("DIRECTORY")},
 
-	{"ac-activate", '\0', POPT_ARG_NONE, &server_ac, 0,
+	{"ac-activate", '\0', 0, G_OPTION_ARG_NONE, &server_ac,
 	 N_("Serve as an ActivationContext (default is as an ObjectDirectory only)"),
 	 NULL},
 
-	{"ior-output-fd", '\0', POPT_ARG_INT, &ior_fd, 0,
+	{"ior-output-fd", '\0', 0, G_OPTION_ARG_INT, &ior_fd,
 	 N_("File descriptor to write IOR to"), N_("FD")},
 
-#ifdef BONOBO_ACTIVATION_DEBUG
-        {"register-server", '0', POPT_ARG_NONE, &server_reg, 0,
-	 "Register as the users' activation server without locking [!]",
+	{"register-server", '\0', 0, G_OPTION_ARG_NONE, &server_reg,
+	 N_("Register as the users' activation server without locking [!] for debugging"),
 	 NULL},
 
-	{"evaluate", '\0', POPT_ARG_STRING, &ac_evaluate, 0,
+#ifdef BONOBO_ACTIVATION_DEBUG
+	{"evaluate", '\0', 0, G_OPTION_ARG_STRING, &ac_evaluate,
 	 N_("Query expression to evaluate"), N_("EXPRESSION")},
 #endif
 
-	POPT_AUTOHELP {NULL}
+        {NULL}
 };
 
 GMainLoop *main_loop = NULL;
@@ -106,35 +200,35 @@ build_src_dir (void)
 
         if (od_source_dir) {
                 g_string_append (real_od_source_dir, od_source_dir);
-                g_string_append_c (real_od_source_dir, ':');
+                g_string_append_c (real_od_source_dir, G_SEARCHPATH_SEPARATOR);
         }
 
         if (env_od_source_dir) {
                 g_string_append (real_od_source_dir,
                                  env_od_source_dir);
-                g_string_append_c (real_od_source_dir, ':');
+                g_string_append_c (real_od_source_dir, G_SEARCHPATH_SEPARATOR);
         }
 
         if (config_file_od_source_dir) {
                 g_string_append (real_od_source_dir,
                                  config_file_od_source_dir);
                 g_free (config_file_od_source_dir);
-                g_string_append_c (real_od_source_dir, ':');
+                g_string_append_c (real_od_source_dir, G_SEARCHPATH_SEPARATOR);
         }
 
         if (gnome_env_od_source_dir) {
-                gnome_dirs = g_strsplit (gnome_env_od_source_dir, ":", -1);
+                gnome_dirs = g_strsplit (gnome_env_od_source_dir, G_SEARCHPATH_SEPARATOR_S, -1);
                 gnome_od_source_dir = g_string_new("");
                 for (i=0; gnome_dirs[i]; i++) {
                         g_string_append (gnome_od_source_dir,
                                          gnome_dirs[i]);
                         g_string_append (gnome_od_source_dir,
-                                         "/lib/bonobo/servers:");
+                                         "/lib/bonobo/servers" G_SEARCHPATH_SEPARATOR_S);
                 }
                 g_strfreev (gnome_dirs);
                 g_string_append (real_od_source_dir,
                                  gnome_od_source_dir->str);
-                g_string_append_c (real_od_source_dir, ':');
+                g_string_append_c (real_od_source_dir, G_SEARCHPATH_SEPARATOR);
         }
 
         g_string_append (real_od_source_dir, SERVERINFODIR);
@@ -148,7 +242,9 @@ log_handler (const gchar *log_domain,
              const gchar *message,
              gpointer user_data)
 {
+#ifdef HAVE_SYSLOG_H
 	int syslog_priority;
+#endif
 	gchar *converted_message;
 
 #ifdef BONOBO_ACTIVATION_DEBUG
@@ -159,6 +255,15 @@ log_handler (const gchar *log_domain,
 		return;
 #endif
 
+	converted_message = g_locale_from_utf8 (message, -1, NULL, NULL, NULL);
+	if (converted_message)
+		message = converted_message;
+
+	if (output_debug)
+                fprintf (stderr, "%s%s", message,
+                         (message[strlen (message) - 1] == '\n' ? "" : "\n"));
+
+#ifdef HAVE_SYSLOG_H
 	/* syslog uses reversed meaning of LEVEL_ERROR and LEVEL_CRITICAL */
 	if (log_level & G_LOG_LEVEL_ERROR)
 		syslog_priority = LOG_CRIT;
@@ -175,14 +280,16 @@ log_handler (const gchar *log_domain,
 	else
 		syslog_priority = LOG_NOTICE;
 
-	converted_message = g_locale_from_utf8 (message, -1, NULL, NULL, NULL);
-	if (converted_message)
-		message = converted_message;
-
 	syslog (syslog_priority, "%s", message);
+#else
+        if (!(log_level & G_LOG_FLAG_FATAL))
+                fprintf (stderr, "%s%s", message,
+                         (message[strlen (message) - 1] == '\n' ? "" : "\n"));
+#endif
 
 	if (log_level & G_LOG_FLAG_FATAL) {
-		fprintf (stderr, "%s", message);
+		fprintf (stderr, "%s%s", message,
+                         (message[strlen (message) - 1] == '\n' ? "" : "\n"));
 		_exit (1);
 	}
 
@@ -194,12 +301,14 @@ redirect_output (int ior_fd)
 {
         int dev_null_fd = -1;
 
-#ifdef BONOBO_ACTIVATION_DEBUG
 	if (output_debug)
 		return dev_null_fd;
-#endif
 
+#ifdef G_OS_WIN32
+	dev_null_fd = open ("NUL:", O_RDWR);
+#else
 	dev_null_fd = open ("/dev/null", O_RDWR);
+#endif
 	if (ior_fd != 0)
 		dup2 (dev_null_fd, 0);
 	if (ior_fd != 1)
@@ -247,16 +356,17 @@ dump_ior (CORBA_ORB orb, int dev_null_fd, CORBA_Environment *ev)
 
 #ifdef BONOBO_ACTIVATION_DEBUG
 	debug_queries ();
+#endif
         if (server_reg) {
                 char *fname;
-                fname = g_strconcat (ORBit_get_safe_tmp (),
-                                     "/bonobo-activation-server-ior", NULL);
-                fh = fopen (fname, "w+");
+                fname = g_build_filename (ORBit_get_safe_tmp (),
+                                          "bonobo-activation-server-ior",
+                                          NULL);
+                fh = g_fopen (fname, "w+");
 		fprintf (fh, "%s\n", ior);
 		fclose (fh);
                 g_free (fname);
         }
-#endif
 
 	CORBA_free (ior);
 }
@@ -264,40 +374,57 @@ dump_ior (CORBA_ORB orb, int dev_null_fd, CORBA_Environment *ev)
 int
 main (int argc, char *argv[])
 {
-        PortableServer_POAManager     poa_manager;
-        PortableServer_POA            root_poa;
+        PortableServer_POA            threaded_poa;
         CORBA_ORB                     orb;
         CORBA_Environment             real_ev, *ev;
         CORBA_Object                  naming_service, existing;
         Bonobo_ActivationEnvironment  environment;
         Bonobo_ObjectDirectory        od;
 	Bonobo_EventSource            event_source;
-        poptContext                   ctx;
+        GOptionContext               *ctx;
         int                           dev_null_fd;
+#ifdef HAVE_SIGACTION
         struct sigaction              sa;
+#endif
         GString                      *src_dir;
+#ifdef HAVE_SYSLOG_H
 	gchar                        *syslog_ident;
+#endif
 	const gchar                  *debug_output_env;
+	GError                       *error = NULL;
 
+#ifdef HAVE_SETSID
         /*
          *    Become process group leader, detach from controlling
          * terminal, etc.
          */
         setsid ();
-        
+#endif
 	/* internationalization. */
 	setlocale (LC_ALL, "");
-        bindtextdomain (PACKAGE, SERVER_LOCALEDIR);
-        textdomain (PACKAGE);
+        bindtextdomain (GETTEXT_PACKAGE, SERVER_LOCALEDIR);
+        textdomain (GETTEXT_PACKAGE);
 
+#ifdef SIGPIPE
         /* Ignore sig-pipe - as normal */
+#ifdef HAVE_SIGACTION
 	memset (&sa, 0, sizeof (sa));
 	sa.sa_handler = SIG_IGN;
 	sigaction (SIGPIPE, &sa, NULL);
+#else
+        signal (SIGPIPE, SIG_IGN);
+#endif
+#endif
 
-	ctx = poptGetContext ("oafd", argc, (const char **)argv, options, 0);
-	while (poptGetNextOpt (ctx) >= 0) ;
-	poptFreeContext (ctx);
+	g_set_prgname ("bonobo-activation-server");
+	ctx = g_option_context_new (NULL);
+	g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+	if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
+		g_printerr ("%s\n", error->message);
+		g_printerr (_("Run '%s --help' to see a full list of available command line options.\n"), g_get_prgname ());
+		exit(1);
+	}
+	g_option_context_free (ctx);
 
         LIBXML_TEST_VERSION;
 	xmlKeepBlanksDefault(0);
@@ -307,24 +434,28 @@ main (int argc, char *argv[])
                 sleep (1);
 #endif
 
+#ifdef HAVE_SYSLOG_H
 	syslog_ident = g_strdup_printf ("bonobo-activation-server (%s-%u)", g_get_user_name (), (guint) getpid ());
 
 	/* openlog does not copy ident string, so we free it on shutdown */
 	openlog (syslog_ident, 0, LOG_USER);
-
-	g_log_set_fatal_mask (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
-	g_log_set_handler (G_LOG_DOMAIN,
-                           G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                           log_handler,
-                           NULL);
+#endif
 
 #ifdef BONOBO_ACTIVATION_DEBUG
 	debug_output_env = g_getenv ("BONOBO_ACTIVATION_DEBUG_OUTPUT");
 	if (debug_output_env && debug_output_env[0] != '\0')
 		output_debug = TRUE;
+#endif
         if (server_reg)
                 output_debug = TRUE;
-#endif
+
+        if (!output_debug) {
+                g_log_set_fatal_mask (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
+                g_log_set_handler (G_LOG_DOMAIN,
+                                   G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+                                   log_handler,
+                                   NULL);
+        }
 
         dev_null_fd = redirect_output (ior_fd);
 
@@ -333,15 +464,17 @@ main (int argc, char *argv[])
 	orb = bonobo_activation_orb_get ();
 	main_loop = g_main_loop_new (NULL, FALSE);
 
+	threaded_poa = server_get_poa ();
+
+        server_lock ();
+
         add_initial_locales ();
         
 	CORBA_exception_init ((ev = &real_ev));
 
-	root_poa = (PortableServer_POA)
-		CORBA_ORB_resolve_initial_references (orb, "RootPOA", ev);
 
         src_dir = build_src_dir ();
-        bonobo_object_directory_init (root_poa, src_dir->str, ev);
+        bonobo_object_directory_init (threaded_poa, src_dir->str, ev);
         g_string_free (src_dir, TRUE);
 
         od = bonobo_object_directory_get ();
@@ -349,11 +482,10 @@ main (int argc, char *argv[])
 
 	memset (&environment, 0, sizeof (Bonobo_ActivationEnvironment));
 
-        /* activate the ORB */
-        poa_manager = PortableServer_POA__get_the_POAManager (root_poa, ev);
-	PortableServer_POAManager_activate (poa_manager, ev);
+        bonobo_activate (); /* activate the ORB */
 
-        naming_service = impl_CosNaming_NamingContext__create (root_poa, ev);
+        /* A non-theading poa - naming almost entirely unused */
+        naming_service = impl_CosNaming_NamingContext__create (bonobo_poa(), ev);
         if (ev->_major != CORBA_NO_EXCEPTION || naming_service == NULL)
                 g_warning ("Failed to create naming service");
         CORBA_exception_init (ev);
@@ -374,21 +506,15 @@ main (int argc, char *argv[])
 	if (existing != CORBA_OBJECT_NIL)
 		CORBA_Object_release (existing, NULL);
 
-        if (ior_fd < 0 && !server_ac
-#ifdef BONOBO_ACTIVATION_DEBUG
-            && !server_reg
-#endif
-            )
+        if (ior_fd < 0 && !server_ac && !server_reg)
                 g_critical ("\n\n-- \nThe bonobo-activation-server must be forked by\n"
                             "libbonobo-activation, and cannot be run itself.\n"
                             "This is due to us doing client side locking.\n-- \n");
 
-#ifdef BONOBO_ACTIVATION_DEBUG
         if (server_reg) {
                 g_warning ("Running in user-forked debugging mode");
                 server_ac = 1;
         }
-#endif
         
         /*
          *     It is no longer useful at all to be a pure
@@ -397,7 +523,7 @@ main (int argc, char *argv[])
          */
         g_assert (server_ac);
         
-        activation_context_setup (root_poa, od, ev);
+        activation_context_setup (threaded_poa, od, ev);
 
         dump_ior (orb, dev_null_fd, ev);
 
@@ -406,19 +532,23 @@ main (int argc, char *argv[])
         if (getenv ("BONOBO_ACTIVATION_DEBUG") == NULL)
                 chdir ("/");
 
+        server_unlock ();
+
 	g_main_loop_run (main_loop);
 
-        nameserver_destroy (root_poa, naming_service, ev);
+        server_lock ();
+
+        nameserver_destroy (bonobo_poa(), naming_service, ev);
         CORBA_Object_release (naming_service, ev);
 
-        bonobo_object_directory_shutdown (root_poa, ev);
+        bonobo_object_directory_shutdown (threaded_poa, ev);
         activation_context_shutdown ();
 
-        CORBA_Object_release ((CORBA_Object) poa_manager, ev);
-        CORBA_Object_release ((CORBA_Object) root_poa, ev);
-
+#ifdef HAVE_SYSLOG_H
 	closelog ();
 	g_free (syslog_ident);
+#endif
+        server_unlock ();
 
 	return !bonobo_debug_shutdown ();
 }
